class DirectedGraph (object):
    """ A model of a directed graph
    
    """
    def __init__(self):
        self.nodes = set ()
        pass

    def add_node (self, n):
        self.nodes.add (n)

    def edge (self, src, dst):
        src.add_succ (dst)

    def get_nodes (self):
        return iter (self.nodes)
    def node_size (self):
        return len (self.nodes)

    def has_node (self, n):
        return n in self.nodes
    
    def __repr__ (self):
        out = ""
        for n in self.nodes:
            for x in n.get_succs ():
                out = out + "{} -> {}\n".format (n, x)
        return out
            

class Node (object):
    """ A model of a node of a directed graph"""
    def __init__ (self, id = None):
        self.succ = []
        self.pred = []
        self.id = id
        pass

    def __repr__ (self):
        if self.id <> None:
            return 'N_{}'.format (self.id)
        else: return object.__repr__ (self)
        

    def has_succ (self, x):
        return x in self.succ
    def has_pred (self, x):
        return x in self.pred

    def add_succ (self, dst):
        if not self.has_succ (dst):
            self.succ.append (dst)
            dst.add_pred (self)

    def add_pred (self, src):
        if not self.has_pred (src):
            self.pred.append (src)
            src.add_succ (self)

    def in_deg (self): return len (self.pred)
    def out_deg (self): return len (self.succ)
    
    def get_succs (self): return iter (self.succ)
    def get_preds (self): return iter (self.preds)


def write_to_dot (graph, file_name, file_type='png'):
    import gv
    
    gv_g = gv.digraph ('graph')
    gv_nodes = dict()
    # nodes
    for n in graph.get_nodes ():
        gv_n = gv_nodes.get (n)
        if gv_n == None: 
            gv_n = gv.node (gv_g, str (n))
            gv_nodes [n] = gv_n
            
    #edges
    for n in graph.get_nodes ():
        for s in n.get_succs ():
            gv.edge (gv_nodes [n], gv_nodes [s])

    gv.layout (gv_g, 'dot')
    gv.render (gv_g, file_type, file_name)


def test ():
    d = DirectedGraph ()
    n1 = Node ()
    n2 = Node ()
    d.add_node (n1)
    d.add_node (n2)
    d.edge (n1, n2)
    
    write_to_dot(d, '/tmp/arie.png')

if __name__ == '__main__':
    test ()

