import argparse

verbose = 0
args = None

def parseArgs (argv):
  argparser = argparse.ArgumentParser(description='Software Proof-based'
                  ' Abstraction with Counterexample-based Refinement')

  # input file
  argparser.add_argument('filename',
                         metavar='file',
                         type=str,
                         help='file containing variable/relation ' +
                         'declarations and horn clauses')
  # print trace for each relation
  argparser.add_argument('-t', '--trace',
                         action='store_true',
                         default=False,
                         help='print the trace for every relation')
  # verbosity
  argparser.add_argument('-v', '--verbose',
                         action='store',
                         type=int,
                         default=0,
                         help='print detailed information')
  # verbosity of z3
  argparser.add_argument('-z', '--z3verbosity',
                         action='store',
                         default=0,
                         type=int,
                         help='Z3 verbosity level (default 0)')
  argparser.add_argument('--z3validate',
                         action='store_true',
                         default=False,
                         help='set validation in z3')
  argparser.add_argument('--vanilla',
                         action='store_true',
                         default=False,
                         help='solve original problem without approximations')
  # preprocessing of rules
  argparser.add_argument('--pp',
                         action='store_true',
                         default=False,
                         help='preprocess rules')
  argparser.add_argument('--dummy_query',
                         action='store_true',
                         default=False,
                         help='make a dummy fp query before creating the hc problem')
  # output file for proof trace
  argparser.add_argument('--proof_out',
                         type=str,
                         default=None,
                         help='output file for proof trace')
  argparser.add_argument('--pba',
                         action='store_true',
                         default=False,
                         help='enable proof-based abstraction')
  argparser.add_argument('--cba',
                         action='store_true',
                         default=False,
                         help='enable cex-based abstraction (i.e. refinement)')
  argparser.add_argument('--refine_naive',
                         action='store_true',
                         default=False,
                         help='eliminate one abstract cex per refinement')
  # initial value for bound consts
  argparser.add_argument('--init_bound',
                         action='store',
                         type=int,
                         default=0,
                         help='initial bound value')
  argparser.add_argument('--bd_increment',
                         action='store',
                         type=int,
                         default=1,
                         help='increment for bd values')
  argparser.add_argument ('--validate',
                          action='store_true',
                          default=False,
                          help='validate every unsat/sat proof')
  argparser.add_argument ('--use_invs',
                          action='store_true',
                          default=False,
                          help='extract and use invariants from' \
                               ' previous fixedpoints')
  argparser.add_argument ('--use_hints',
                          action='store_true',
                          default=False,
                          help='use hints')
  argparser.add_argument ('--no_sig_abs', action='store_true', default=False,
                          help='Do no signature abstraction')
  argparser.add_argument ('--no_trans_abs', action='store_true', default=False,
                          help='Do no transition relation abstraction')
  argparser.add_argument ('--conc_abs', action='store_true', default=False,
                          help='Start with concrete system when abstracting')
  argparser.add_argument ('--fix_lemmas', action='store_true', default=False,
                          help='Fix lemmas by asserting them during abstraction')

  argparser.add_argument ('--no_bounds', action='store_true', default=False,
                          help='Ignore bounds. Used for testing.')
  argparser.add_argument('--max_refine',
                         action='store',
                         type=int,
                         default=0,
                         help='concretize after these many refinements')
  argparser.add_argument ('--bound_lemmas', action='store_true', default=False,
                          help='Add lemmas to bound counters. (Breaks validation!)')
  argparser.add_argument ('--z3_trace', default=None,
                          help='Enable z3 trace')
  argparser.add_argument ('--up-counter', dest='up_counter', 
                          action='store_true', default=False, 
                          help='Use upward counter for bounding loops')
  argparser.add_argument ('--refine-use-mus', dest='refine_use_mus',
                          default=False, action='store_true',
                          help='Use MUS in refine instead of simple UNSAT core')

  argparser.add_argument ('--down',
                          default=False, action='store_true',
                          help='bound using downward counters with pba and cba')
  argparser.add_argument ('--sv-down', dest='sv_down',
                          default=False, action='store_true',
                          help='like --down, with state-variable abstraction')
  argparser.add_argument ('--up',
                          default=False, action='store_true',
                          help='bound using upward counters with pba and cba')
  argparser.add_argument ('--pba-down', dest='pba_down',
                          default=False, action='store_true',
                          help='bound using downward counters with pba only')
  argparser.add_argument ('--pba-up', dest='pba_up',
                          default=False, action='store_true',
                          help='bound using upward counters with pba only')
  argparser.add_argument ('--z3down',
                          default=False, action='store_true',
                          help='no abstraction with extra dummy constraints for'
                          ' downward counters')
  argparser.add_argument ('--z3up',
                          default=False, action='store_true',
                          help='no abstraction with extra dummy constraints for'
                          ' upward counters')

  # get cmd line arguments
  cmd_args = argparser.parse_args (argv)

  if cmd_args.down:
    cmd_args.pba = True
    cmd_args.cba = True
    cmd_args.use_invs = True
    cmd_args.use_hints = True

    cmd_args.conc_abs = True
    cmd_args.no_sig_abs = False
    cmd_args.no_trans_abs = False

    cmd_args.init_bound = 0
    cmd_args.bd_increment = 1

    cmd_args.vanilla = False
    cmd_args.fix_lemmas = False
    cmd_args.no_bounds = False
    cmd_args.bound_lemmas = False
    cmd_args.up_counter = False
    cmd_args.refine_use_mus = False

  elif cmd_args.up:
    cmd_args.pba = True
    cmd_args.cba = True
    cmd_args.use_invs = True
    cmd_args.use_hints = True

    cmd_args.conc_abs = True
    cmd_args.no_sig_abs = False
    cmd_args.no_trans_abs = False

    cmd_args.init_bound = 1
    cmd_args.bd_increment = 1

    cmd_args.vanilla = False
    cmd_args.fix_lemmas = False
    cmd_args.no_bounds = False
    cmd_args.bound_lemmas = False
    cmd_args.up_counter = True
    cmd_args.refine_use_mus = True

  elif cmd_args.pba_up:
    cmd_args.pba = True
    cmd_args.cba = False
    cmd_args.use_invs = True
    cmd_args.use_hints = True

    cmd_args.conc_abs = True
    cmd_args.no_sig_abs = False
    cmd_args.no_trans_abs = False

    cmd_args.init_bound = 1
    cmd_args.bd_increment = 1

    cmd_args.vanilla = False
    cmd_args.fix_lemmas = False
    cmd_args.no_bounds = False
    cmd_args.bound_lemmas = False
    cmd_args.up_counter = True
    cmd_args.refine_use_mus = True

  elif cmd_args.sv_down:
    cmd_args.pba = True
    cmd_args.cba = True
    cmd_args.use_invs = True
    cmd_args.use_hints = True

    cmd_args.conc_abs = True
    cmd_args.no_sig_abs = False
    cmd_args.no_trans_abs = True

    cmd_args.init_bound = 0
    cmd_args.bd_increment = 1

    cmd_args.vanilla = False
    cmd_args.fix_lemmas = False
    cmd_args.no_bounds = False
    cmd_args.bound_lemmas = False
    cmd_args.up_counter = False
    cmd_args.refine_use_mus = False

  elif cmd_args.pba_down:
    cmd_args.pba = True
    cmd_args.cba = False
    cmd_args.use_invs = True
    cmd_args.use_hints = True

    cmd_args.conc_abs = True
    cmd_args.no_sig_abs = False
    cmd_args.no_trans_abs = False

    cmd_args.init_bound = 0
    cmd_args.bd_increment = 1

    cmd_args.vanilla = False
    cmd_args.fix_lemmas = False
    cmd_args.no_bounds = False
    cmd_args.bound_lemmas = False
    cmd_args.up_counter = False
    cmd_args.refine_use_mus = False

  elif cmd_args.z3down:
    cmd_args.vanilla = False
    cmd_args.no_bounds = True

  elif cmd_args.z3up:
    cmd_args.pba = True
    cmd_args.cba = True
    cmd_args.use_invs = True
    cmd_args.use_hints = True

    cmd_args.conc_abs = True
    cmd_args.no_sig_abs = False
    cmd_args.no_trans_abs = False

    cmd_args.init_bound = 1
    cmd_args.bd_increment = 1

    cmd_args.vanilla = False
    cmd_args.fix_lemmas = False
    cmd_args.no_bounds = True
    cmd_args.bound_lemmas = False
    cmd_args.up_counter = True
    cmd_args.refine_use_mus = True


  global verbose
  verbose = cmd_args.verbose

  global args
  args = cmd_args
  return args
