from graph import *



class WtoComponent (list):
    """ A component in WTO """
    def __init__ (self, data=[]):
        fst = data [0]
        if isinstance (fst, WtoComponent):
            list.__init__ (self, fst)
            self.extend (data [1:])
        else:
            list.__init__ (self, data)
    def __getslice__ (self, i, j):
        return WtoComponent (list.__getslice__(self, i, j))

    def __repr__ (self):
        return 'C{}'.format (list.__repr__ (self))


class WtoWalkStopFlag:
    def __init__ (self):
        self.Stop = False

    def stop (self):
        self.Stop = True

def wto_is_head (comp, elmt):
    return len (comp) > 0 and comp [0] == elmt
def wto_is_last (comp, elmt): 
    return len (comp) > 0 and comp[-1] == elmt

def wto_inf_walk (wto, comps = []):
    """ Infinite wto walk. Keeps iterating a component until it is
    popped off by the client """
    depth = len (comps)
    while True:
        for elmt in wto:
            if isinstance (elmt, WtoComponent):
                comps.append (elmt)
                for xcomps, xelmt in wto_inf_walk (elmt, comps):
                    yield (xcomps, xelmt)
            else:
                yield (comps, elmt)

            if len (comps) < depth: break

        if not isinstance (wto, WtoComponent): break
        if len (comps) < depth: break

def wto_inf_walk_old (wto, comps = [], spf = WtoWalkStopFlag ()):
    # only do one iteration if wto is not a component
    if not isinstance (wto, WtoComponent): spf.stop ()
    while True:
        for elmt in wto:
            if isinstance (elmt, WtoComponent):
                comps.append (elmt)
                for xspf, xcomps, xelmt in \
                        wto_inf_walk (elmt, comps, WtoWalkStopFlag ()):
                    yield (xspf, xcomps, xelmt)
                comps.pop ()
            else:
                yield (spf, comps, elmt)
        if spf.Stop: break

def wto_walk (wto, heads = []):
    """ A walk over WTO. Returns an iterator that yields a pair (h, x)
    such that h are the components to which x belongs """ 
    assert len (wto) > 0
    for elmt in wto:
        if isinstance (elmt, WtoComponent):
            assert len (elmt) > 0
            heads.append (elmt)
            for heads, xelmt in wto_walk (elmt, heads):
                yield (heads, xelmt)
            heads.pop ()
        else:
            yield (heads, elmt)
    
def topo_walk (wto):
    """ Topological Walk over a WTO """
    class TopoIter (object):
        def __init__ (self, wtoWalk):
            self.walk = wtoWalk
        def __iter__ (self):
            return self
        def next (self):
            return self.walk.next ()[1]
    return TopoIter (wto_walk (wto))

def wto_pp (wto, pp=str):
    out = ""
    for c, x in wto_walk (wto):
        if len(c) > 0 and x == c[-1][0]: out += "(" 
        out += ' ' + pp(x) + ' '
        if len(c) > 0 and x == c[-1][-1]: out += ")"
    return out
        
def test_wto_walk ():
    print 'BEGIN: Test WTO Walk'
    W = WtoComponent
    w = [1, W([2, W([3, 4]), 5]), 6]
    print 'WTO:', w
    for h, x in wto_walk (w):
        print h, 'and', x
            
    print list (topo_walk (w))

    # number of times a head was reached
    bounds = dict()
    for c, x in wto_inf_walk (w):
        # if at head
        if len(c) > 0 and wto_is_head (c[-1], x): 
            # update bound
            bounds [x] = bounds.get(x, 0) + 1
            # if saturated, indicate exit by popping current component
            if bounds [x] > 3: 
                bounds [x] = 0
                c.pop ()
                continue
            print '(',

        print x, ' ', 
        
        if len(c) > 0 and wto_is_last (c[-1], x):  print ')',
    print
    print 'END: Test WTO Walk'

""" Computes Weak Topological Order (WTO) of a given graph rooted at v """
def wto (graph, root):
    (ig, ir) = init_itv_graph (graph, root)

    ### reduce the graph until single node remains
    while ig.node_size () <> 1:
        old_size = ig.node_size ()
        (ig, ir) = reduce_itv_graph (ig, ir)

        ### reduce failed -- non-reducible graph
        if ig.node_size () >= old_size:
            raise NonReducibleFlowGraph ()
            
    return ir.wto

def init_itv_graph (graph, root):
    ig = DirectedGraph ()
    ir = ItvNode (root)
    
    seen = dict()
    for n in graph.get_nodes ():
        itv_n = itv_node (ig, n, seen)

        if n == root: ir = itv_n
        for x in n.get_succs ():
            ### exclude self-loops
            if x <> n:
                ig.edge (itv_n, itv_node (ig, x, seen))
    return (ig, ir)

def itv_node (ig, n, seen):
    itv_n = seen.get (n)
    if itv_n == None:
        itv_n = ItvNode (n)
        seen [n] = itv_n
        ig.add_node (itv_n)
    return itv_n
    

def reduce_itv_graph (g, r):
    return ReduceItvGraph (g, r).reduce ()

class ReduceItvGraph (object):
    def __init__ (self, graph, root):
        self.g = graph
        self.r = root
        self.ig = DirectedGraph ()

        from collections import deque
        self.todo = deque ()
        self.itv_nodes = dict ()

    def reduce (self):
        self.todo.append (self.r)
        while len (self.todo) <> 0:
            v = self.todo.popleft ()
            self.compute_itv (v)

        return (self.ig, self.get_itv_node (self.r))

    def get_itv_node (self, n):
        itv_node = self.itv_nodes.get (n)

        if itv_node == None:
            itv_node = ItvNode ()
            itv_node.wto = n.wto
            self.ig.add_node (itv_node)
            self.itv_nodes [n] = itv_node

        return itv_node
    
    def new_head (self, head, src):
        new_h = self.get_itv_node (head)
        assert src <> new_h
        self.ig.edge (src, new_h)
        self.todo.append (head)

    
    def compute_itv (self, v):
        self.local_heads = set ()
        itv_node = self.get_itv_node (v)
        
        if self.compute_itv_rec (itv_node, v.get_succs ()):
            itv_node.wto = WtoComponent (itv_node.wto)
        
        # all local heads are true heads that have to be processed
        for h in self.local_heads: self.new_head (h, itv_node)


    def compute_itv_rec (self, itv, ss):
        retval = False
        for s in ss:

            s_itv_node = self.itv_nodes.get (s)
            # back-edge to the head of the component
            if s_itv_node  == itv:
                retval = True
                continue

            # s is a head of another interval
            if s_itv_node <> None:  
                assert itv <> s_itv_node
                self.ig.edge (itv, s_itv_node)
                continue
        
            s.mark ()
            if s.vcount < s.in_deg ():
                self.local_heads.add(s)
            else:
                self.add_to_itv (itv, s)
                retval = self.compute_itv_rec (itv, s.get_succs ()) or retval
        return retval

    def add_to_itv (self, itv, n):
        if n in self.local_heads: self.local_heads.remove (n)
        if isinstance (n.wto, WtoComponent):
            itv.wto.append (n.wto)
        else:
            itv.wto.extend (n.wto)

class ItvNode (Node):
    def __init__(self, n = None):
        Node.__init__ (self)
        self.vcount = 0

        if n <> None:
            if n.has_succ (n): W = WtoComponent
            else: W = list
            self.wto = W ([n])
            

    def mark (self): self.vcount = self.vcount + 1
    def __repr__ (self):
        return "ITV({})".format (self.wto)

        
class NonReducibleFlowGraph (BaseException):
    pass

def test_wto ():
    print 'BEGIN test_wto()'
    g = DirectedGraph ()
    nodes = [Node (i) for i in range (0, 4)]
    for i in range (0, 4): g.add_node (nodes [i])
    
    g.edge (nodes [0], nodes [1])
    g.edge (nodes [1], nodes [1])
    g.edge (nodes [1], nodes [2])
    g.edge (nodes [2], nodes [1])
    g.edge (nodes [2], nodes [2])
    g.edge (nodes [2], nodes [3])

    print 'Graph'
    print g

    print 'WTO'
    w = wto (g, nodes [0])
    print wto_pp (w)
    print 'END test_wto()'

if __name__ == '__main__':
    test_wto_walk ()
    test_wto ()
